/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <app.h>
#include <Elementary.h>
#include <viewmgr.h>
#include <app_debug.h>
#include <app_define.h>

#include "define.h"
#include "notification_view.h"
#include "square_view.h"
#include "action_menu_view.h"

#define KEY_LAUNCH_TYPE "launch_type"
#define TYPE_SQUARE "infosquare"

SET_TAG(PACKAGE)

struct _appdata {
	const char *name;
	Evas_Object *win;
};

static Evas_Object *_add_win(const char *name)
{
	Evas_Object *win;

	win = elm_win_add(NULL, name, ELM_WIN_BASIC);
	if (!win) {
		_ERR("failed to create win");
		return NULL;
	}
	elm_win_alpha_set(win, EINA_FALSE);
	elm_win_focus_highlight_enabled_set(win, EINA_TRUE);
	elm_win_focus_highlight_style_set(win, "invisible");

	evas_object_show(win);

	return win;
}

static void _pause(void *data)
{
}

static void _resume(void *data)
{
}

static bool _create(void *data)
{
	struct _appdata *ad;
	Evas_Object *win;

	if (!data) {
		_ERR("failed to get data");
		return false;
	}

	ad = data;

	elm_app_base_scale_set(APP_BASE_SCALE);
	elm_theme_overlay_add(NULL, THEMEFILE);
	elm_config_focus_move_policy_set(ELM_FOCUS_MOVE_POLICY_CLICK);

	win = _add_win(ad->name);
	if (!win) {
		_ERR("failed to create win object");
		return false;
	}

	if (!viewmgr_create(win)) {
		_ERR("failed to initialize viewmgr");
		evas_object_del(win);
		return false;
	}

	viewmgr_add_view(notification_view_get_vclass(), NULL);
	viewmgr_add_view(square_view_get_vclass(), NULL);
	viewmgr_add_view(action_menu_view_get_vclass(), NULL);

	ad->win = win;

	return true;
}

static void _terminate(void *data)
{
	struct _appdata *ad;

	if (!data) {
		_ERR("failed to get data");
		return;
	}

	ad = data;

	viewmgr_remove_view(NOTIFICATION_VIEW);
	viewmgr_remove_view(SQUARE_VIEW);
	viewmgr_remove_view(ACTION_MENU_VIEW);
	viewmgr_destroy();

	if (ad->win) {
		evas_object_del(ad->win);
		ad->win = NULL;
	}
}

static void _control(app_control_h control, void *data)
{
	struct _appdata *ad;
	int r;
	char *type = NULL;

	if (!data) {
		_ERR("failed to get data");
		return;
	}

	ad = data;

	if (ad->win)
		elm_win_activate(ad->win);

	r = app_control_get_extra_data(control, KEY_LAUNCH_TYPE, &type);
	if (r == APP_CONTROL_ERROR_NONE &&
			type && !strcmp(type, TYPE_SQUARE))
		viewmgr_push_view(SQUARE_VIEW);
	else
		viewmgr_push_view(NOTIFICATION_VIEW);
}

int main(int argc, char *argv[])
{
	struct _appdata ad;
	ui_app_lifecycle_callback_s cbs = {
		.create = _create,
		.terminate = _terminate,
		.pause = _pause,
		.resume = _resume,
		.app_control = _control,
	};

	memset(&ad, 0x00, sizeof(ad));
	ad.name = PACKAGE;

	return ui_app_main(argc, argv, &cbs, &ad);
}
