/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <viewmgr.h>
#include <inputmgr.h>
#include <app_debug.h>
#include <notification.h>
#include <notification_list.h>
#include <notification_noti.h>
#include <notification_internal.h>
#include <app.h>

#include "define.h"
#include "util.h"

#define STYLE_CATEGORY_TITLE "category_btn"
#define STYLE_GRID_ITEM "notification_item"

#define NOTIFICATION_CATEGORY_PADDING_X 62
#define NOTIFICATION_CATEGORY_PADDING_Y 0
#define NOTIFICATION_CATEGORY_WIDTH (1920 - NOTIFICATION_CATEGORY_PADDING_X)
#define NOTIFICATION_CATEGORY_HEIGHT 614
#define NOTIFICATION_CATEGORY_STEP_SIZE 70
#define NOTIFICATION_ITEM_WIDTH 489
#define NOTIFICATION_ITEM_HEIGHT 134
#define NOTIFICATION_ITEM_PADDING 26
#define NOTIFICATION_ITEMS_IN_COL 4.0

enum _obj_id {
	OBJ_CLEAR,
	OBJ_TITLE,
	OBJ_GRID
};

enum notification_category {
	CATEGORY_FIRST = 0,
	CATEGORY_TODAY = CATEGORY_FIRST,
	CATEGORY_ONGOING,
	CATEGORY_TV,
	CATEGORY_COMMUNICATION,
	CATEGORY_EVENT,
	CATEGORY_THINGS,
	CATEGORY_INFORMATION,
	CATEGORY_FILE,
	CATEGORY_MAX
};

struct _category {
	Evas_Object *title;
	Evas_Object *grid;
};

struct _priv {
	Evas_Object *base;
	Evas_Object *clear_btn;
	Evas_Object *category_container;
	Elm_Object_Item *current_it;
	Elm_Gengrid_Item_Class *grid_class;
	struct _category *category[CATEGORY_MAX];
};

static void _base_key_up_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Up *ev);
static void _clear_key_up_cb(int id, void *data,
		Evas *e, Evas_Object *obj, Evas_Event_Key_Up *ev);
static void _clear_mouse_up_cb(int id, void *data,
		Evas *e, Evas_Object *obj, Evas_Event_Mouse_Up *ev);
static void _category_unfocused_cb(int id, void *data,
		Evas_Object *obj, Elm_Object_Item *it);
static void _mouse_move(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev);
static void _focus_key_up_cb(int id, void *data,
		Evas *e, Evas_Object *obj, Evas_Event_Key_Up *ev);
static void _focus_focused_cb(int id, void *data,
		Evas_Object *obj, Elm_Object_Item *it);
static void _focus_unfocused_cb(int id, void *data,
		Evas_Object *obj, Elm_Object_Item *it);

static input_handler base_handler = {
	.key_up = _base_key_up_cb,
};
static input_handler clear_handler = {
	.key_up = _clear_key_up_cb,
	.mouse_up = _clear_mouse_up_cb
};
static input_handler category_handler = {
	.unfocused = _category_unfocused_cb,
};
static input_handler focus_handler = {
	.mouse_move = _mouse_move,
	.key_up = _focus_key_up_cb,
	.focused = _focus_focused_cb,
	.unfocused = _focus_unfocused_cb
};

static const char *_get_category_title(int category)
{
	switch (category) {
	case CATEGORY_TODAY:
		return STR_CATEGORY_TODAY;
	case CATEGORY_ONGOING:
		return STR_CATEGORY_ONGOING;
	case CATEGORY_TV:
		return STR_CATEGORY_TV;
	case CATEGORY_COMMUNICATION:
		return STR_CATEGORY_COMMUNICATION;
	case CATEGORY_EVENT:
		return STR_CATEGORY_EVENT;
	case CATEGORY_THINGS:
		return STR_CATEGORY_THINGS;
	case CATEGORY_INFORMATION:
		return STR_CATEGORY_INFORMATION;
	case CATEGORY_FILE:
		return STR_CATEGORY_FILE;
	default:
		return NULL;
	}
}

static void _show_popup(void *data)
{
	viewmgr_update_view(NOTIFICATION_VIEW,
			UPDATE_TYPE_SHOW_POPUP, data);
}

static void _grid_select(void *data, Evas_Object *obj, void *event_info)
{
	notification_h noti;

	noti = elm_object_item_data_get(event_info);
	if (!noti)
		return;

	ecore_job_add(_show_popup, noti);
}

static Evas_Object *_add_category_title(Evas_Object *base, int category)
{
	Evas_Object *btn;

	btn = elm_button_add(base);
	if (!btn) {
		_ERR("failed to create button object");
		return NULL;
	}

	elm_object_style_set(btn, STYLE_CATEGORY_TITLE);
	elm_object_text_set(btn, _get_category_title(category));
	elm_object_focus_allow_set(btn, EINA_TRUE);

	elm_object_part_content_set(base,
			PART_NOTIFICATION_CATEGORY_TITLE, btn);

	return btn;
}

static Evas_Object *_add_category_grid(Evas_Object *base)
{
	Evas_Object *grid;

	grid = elm_gengrid_add(base);
	if (!grid) {
		_ERR("failed to create grid object");
		return NULL;
	}

	elm_gengrid_item_size_set(grid,
			ELM_SCALE_SIZE(NOTIFICATION_ITEM_WIDTH +
					NOTIFICATION_ITEM_PADDING),
			ELM_SCALE_SIZE(NOTIFICATION_ITEM_HEIGHT +
					NOTIFICATION_ITEM_PADDING));
	elm_gengrid_align_set(grid, 0.0, 0.0);
	elm_gengrid_horizontal_set(grid, EINA_TRUE);
	evas_object_size_hint_weight_set(grid,
			EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_gengrid_select_mode_set(grid, ELM_OBJECT_SELECT_MODE_ALWAYS);

	elm_object_content_set(base, grid);

	return grid;
}

static void _draw_category(struct _priv *priv, int category)
{
	Evas_Object *ly, *title, *grid;

	if (category < CATEGORY_FIRST ||
			category >= CATEGORY_MAX) {
		_ERR("Undefined category");
		return;
	}

	priv->category[category] =
			calloc(1, sizeof(*priv->category[category]));

	if (!priv->category[category]) {
		_ERR("failed to allocate category");
		return;
	}

	ly = elm_layout_add(priv->category_container);
	if (!ly) {
		_ERR("failed to create category layout");
		goto error;
	}
	elm_layout_file_set(ly, EDJEFILE, GRP_NOTIFICATION_CATEGORY);
	evas_object_size_hint_align_set(ly, 0.0, 0.0);

	title = _add_category_title(ly, category);
	if (!title) {
		_ERR("failed to create title");
		goto error;
	}

	grid = _add_category_grid(ly);
	if (!grid) {
		_ERR("failed to create grid");
		goto error;
	}

	elm_object_focus_next_object_set(title, priv->clear_btn, ELM_FOCUS_UP);
	elm_object_focus_next_object_set(grid, title, ELM_FOCUS_UP);
	elm_object_focus_next_object_set(grid, grid, ELM_FOCUS_RIGHT);

	inputmgr_add_callback(title, category, &category_handler, priv);
	inputmgr_add_callback(title, OBJ_TITLE, &focus_handler, priv);
	inputmgr_add_callback(grid, category, &category_handler, priv);
	inputmgr_add_callback(grid, OBJ_GRID, &focus_handler, priv);

	elm_box_pack_end(priv->category_container, ly);
	evas_object_show(grid);
	evas_object_show(ly);

	priv->category[category]->title = title;
	priv->category[category]->grid = grid;

	return;

error:
	if (priv->category[category])
		free(priv->category[category]);
	if (ly)
		evas_object_del(ly);
}

static void _draw_notification(struct _priv *priv, int category,
		notification_list_h ongoing_list, notification_list_h noti_list)
{
	notification_h noti;
	Elm_Object_Item *it;
	int count = 0, col, w;

	if (category < CATEGORY_FIRST ||
			category >= CATEGORY_MAX) {
		_ERR("Undefined category");
		return;
	}

	if (!priv->category[category]) {
		_ERR("failed to get category");
		return;
	}

	elm_gengrid_clear(priv->category[category]->grid);

	while (ongoing_list != NULL) {
		noti = notification_list_get_data(ongoing_list);
		it = elm_gengrid_item_append(priv->category[category]->grid,
				priv->grid_class, noti, _grid_select, priv);
		elm_object_item_data_set(it, noti);

		ongoing_list = notification_list_remove(ongoing_list, noti);
		count++;
	}

	while (noti_list != NULL) {
		noti = notification_list_get_data(noti_list);
		it = elm_gengrid_item_append(priv->category[category]->grid,
				priv->grid_class, noti, _grid_select, priv);
		elm_object_item_data_set(it, noti);

		noti_list = notification_list_remove(noti_list, noti);
		count++;
	}

	col = ceil(count / NOTIFICATION_ITEMS_IN_COL);
	w = col * (NOTIFICATION_ITEM_WIDTH + NOTIFICATION_ITEM_PADDING);
	if (w > NOTIFICATION_CATEGORY_WIDTH)
		w = NOTIFICATION_CATEGORY_WIDTH;
	evas_object_size_hint_min_set(priv->category[category]->grid,
			ELM_SCALE_SIZE(w),
			ELM_SCALE_SIZE(NOTIFICATION_CATEGORY_HEIGHT +
			NOTIFICATION_ITEM_PADDING));
}

static void _update_list(struct _priv *priv)
{
	notification_list_h ongoing_list = NULL, noti_list = NULL;
	Elm_Object_Item *it;

	notification_get_list(NOTIFICATION_TYPE_NOTI, -1, &noti_list);
	notification_get_list(NOTIFICATION_TYPE_ONGOING, -1, &ongoing_list);
	if (!ongoing_list && !noti_list) {
		elm_object_signal_emit(priv->base,
				SIGNAL_NO_NOTIFICATION, SOURCE_PROGRAM);
		elm_object_disabled_set(priv->clear_btn, EINA_TRUE);
		elm_object_focus_allow_set(priv->base, EINA_TRUE);
		elm_object_focus_set(priv->base, EINA_TRUE);

		return;
	}

	elm_object_signal_emit(priv->base,
			SIGNAL_NOTIFICATION, SOURCE_PROGRAM);
	elm_object_disabled_set(priv->clear_btn, EINA_FALSE);
	elm_object_focus_allow_set(priv->base, EINA_FALSE);

	if (!priv->category[CATEGORY_TODAY])
		_draw_category(priv, CATEGORY_TODAY);

	_draw_notification(priv, CATEGORY_TODAY, ongoing_list, noti_list);

	if (priv->category[CATEGORY_TODAY]) {
		it = elm_gengrid_first_item_get(
				priv->category[CATEGORY_TODAY]->grid);
		if (it)
			elm_object_item_focus_set(it, EINA_TRUE);
	}

	util_set_last_viewed();
}

static char *_grid_label_get(void *data,
		Evas_Object *obj,
		const char *part)
{
	notification_h noti = data;
	char *buf = NULL;

	if (!noti)
		return NULL;

	if (!strcmp(part, PART_TITLE)) {
		notification_get_text(noti, NOTIFICATION_TEXT_TYPE_TITLE, &buf);
		if (!buf)
			notification_get_pkgname(noti, &buf);

		return buf ? strdup(buf) : NULL;
	} else if (!strcmp(part, PART_TEXT1)) {
		notification_get_text(noti,
				NOTIFICATION_TEXT_TYPE_CONTENT, &buf);

		return buf ? strdup(buf) : NULL;
	}

	return NULL;
}

static Evas_Object *_grid_content_get(void *data,
		Evas_Object *obj,
		const char *part)
{
	notification_h noti = data;
	char *icon_path = NULL;

	if (!noti)
		return NULL;

	if (!strcmp(part, PART_ICON)) {
		notification_get_image(noti,
				NOTIFICATION_IMAGE_TYPE_ICON,
				&icon_path);

		if (!icon_path)
			return NULL;

		Evas_Object *icon = elm_image_add(obj);

		elm_image_file_set(icon, icon_path, NULL);
		evas_object_show(icon);

		return icon;
	}

	return NULL;
}

static void _grid_del(void *data, Evas_Object *obj)
{
	notification_h noti = data;

	if (!noti)
		return;

	notification_free(noti);
}

static void _notification_changed_cb(void *data, notification_type_e type)
{
	struct _priv *priv = data;

	if (priv)
		_update_list(priv);
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	Evas_Object *scroller;

	if (!win) {
		_ERR("failed to get win object");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to allocate priv");
		return NULL;
	}

	priv->base = elm_layout_add(win);
	if (!priv->base) {
		_ERR("failed to create base object");
		free(priv);
		return NULL;
	}
	elm_layout_file_set(priv->base, EDJEFILE, GRP_NOTIFICATION_VIEW);
	evas_object_size_hint_weight_set(priv->base,
			EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, priv->base);
	inputmgr_add_callback(priv->base, 0, &base_handler, NULL);

	scroller = elm_scroller_add(priv->base);
	evas_object_size_hint_align_set(scroller, 0.0, 0.0);
	elm_object_content_set(priv->base, scroller);
	elm_scroller_step_size_set(scroller,
			ELM_SCALE_SIZE(NOTIFICATION_CATEGORY_STEP_SIZE), 0);

	priv->category_container = elm_box_add(scroller);
	elm_object_content_set(scroller, priv->category_container);
	elm_box_align_set(priv->category_container, 0.0, 0.0);
	elm_box_horizontal_set(priv->category_container, EINA_TRUE);
	elm_box_padding_set(priv->category_container,
			ELM_SCALE_SIZE(NOTIFICATION_CATEGORY_PADDING_X),
			ELM_SCALE_SIZE(NOTIFICATION_CATEGORY_PADDING_Y));
	elm_object_focus_allow_set(scroller, EINA_FALSE);
	elm_object_focus_allow_set(priv->category_container, EINA_FALSE);

	priv->clear_btn = elm_layout_add(priv->base);
	elm_layout_file_set(priv->clear_btn, EDJEFILE, GRP_NOTIFICATION_CLEAR);
	elm_object_focus_allow_set(priv->clear_btn, EINA_TRUE);
	elm_object_part_content_set(priv->base,
			PART_NOTIFICATION_CLEAR, priv->clear_btn);
	evas_object_show(priv->clear_btn);
	inputmgr_add_callback(priv->clear_btn, 0, &clear_handler, NULL);
	inputmgr_add_callback(priv->clear_btn, OBJ_CLEAR, &focus_handler, priv);

	priv->grid_class = elm_gengrid_item_class_new();
	if (!priv->grid_class) {
		_ERR("failed to create gengrid item class");
		free(priv->base);
		free(priv);
		return NULL;
	}
	priv->grid_class->item_style = STYLE_GRID_ITEM;
	priv->grid_class->func.text_get = _grid_label_get;
	priv->grid_class->func.content_get = _grid_content_get;
	priv->grid_class->func.state_get = NULL;
	priv->grid_class->func.del = _grid_del;

	viewmgr_set_view_data(NOTIFICATION_VIEW, priv);

	return priv->base;
}

static void _show(void *view_data)
{
	struct _priv *priv;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = (struct _priv *) view_data;

	evas_object_show(priv->base);

	_update_list(priv);

	notification_resister_changed_cb(_notification_changed_cb, priv);
}

static void _hide(void *view_data)
{
	struct _priv *priv;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = (struct _priv *) view_data;

	notification_unresister_changed_cb(_notification_changed_cb);

	evas_object_hide(priv->base);
}

static void _destroy(void *view_data)
{
	struct _priv *priv;
	int i;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = (struct _priv *) view_data;

	notification_unresister_changed_cb(_notification_changed_cb);

	evas_object_del(priv->base);

	elm_gengrid_item_class_free(priv->grid_class);

	for (i = CATEGORY_FIRST; i < CATEGORY_MAX; i++) {
		if (priv->category[i])
			free(priv->category[i]);
	}

	free(priv);
}

static void _update(void *view_data, int type, void *data)
{
	struct _priv *priv;
	notification_h noti;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	switch (type) {
	case UPDATE_TYPE_SHOW_POPUP:
		noti = data;

		if (!noti)
			return;

		util_show_notification_popup(priv->base, noti, false);
		break;
	}
}

static view_class vclass = {
	.view_id = NOTIFICATION_VIEW,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
	.update = _update,
};

static void _base_key_up_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Up *ev)
{
	if (!strcmp(ev->keyname, KEY_BACK) ||
			!strcmp(ev->keyname, KEY_ESC)) {
		ui_app_exit();
	}
}

static void _clear_key_up_cb(int id, void *data, Evas *e,
		Evas_Object *obj, Evas_Event_Key_Up *ev)
{
	if (!strcmp(ev->keyname, KEY_ENTER))
		util_clear_notification();
}

static void _clear_mouse_up_cb(int id, void *data, Evas *e,
		Evas_Object *obj, Evas_Event_Mouse_Up *ev)
{
	util_clear_notification();
}

static void _category_unfocused_cb(int id, void *data,
		Evas_Object *obj, Elm_Object_Item *it)
{
	struct _priv *priv;

	if (!data) {
		_ERR("failed to get data");
		return;
	}

	priv = data;

	if (priv->category[id])
		elm_object_focus_next_object_set(priv->clear_btn,
				priv->category[id]->title, ELM_FOCUS_DOWN);
}

static void _mouse_move(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev)
{
	Elm_Object_Item *it;

	switch (id) {
	case OBJ_GRID:
		it = elm_gengrid_at_xy_item_get(obj, ev->cur.canvas.x,
				ev->cur.canvas.y, NULL, NULL);

		if (!it)
			return;

		if (!elm_object_item_focus_get(it))
			elm_object_item_focus_set(it, EINA_TRUE);
		break;
	case OBJ_CLEAR:
	case OBJ_TITLE:
		elm_object_focus_set(obj, EINA_TRUE);
		break;
	}
}

static void _focus_key_up_cb(int id, void *data, Evas *e,
		Evas_Object *obj, Evas_Event_Key_Up *ev)
{
}

static void _focus_focused_cb(int id, void *data,
		Evas_Object *obj, Elm_Object_Item *it)
{
	struct _priv *priv;

	if (!data || !it)
		return;

	priv = data;

	priv->current_it = it;
}

static void _focus_unfocused_cb(int id, void *data,
		Evas_Object *obj, Elm_Object_Item *it)
{
	struct _priv *priv;

	if (!data || !it)
		return;

	priv = data;

	if (priv->current_it == it)
		priv->current_it = NULL;
}

view_class *notification_view_get_vclass(void)
{
	return &vclass;
}
