/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <app_debug.h>
#include <notification.h>
#include <notification_list.h>
#include <inputmgr.h>
#include <app_contents.h>

#include "define.h"

#define STYLE_POPUP_UP_BTN "notification/popup_up_btn"
#define STYLE_POPUP_DOWN_BTN "notification/popup_down_btn"
#define STYLE_POPUP_OK_BTN "notification/popup_ok_btn"

enum _button_id {
	BUTTON_OK,
	BUTTON_OK_NOTI_DEL,
	BUTTON_CANCEL,
};

static void _key_up_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Up *ev);
static void _clicked(int id, void *data, Evas_Object *obj);

static void _mouse_move(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev);

static input_handler popup_handler = {
	.key_up = _key_up_cb,
	.clicked = _clicked,
};

static input_handler focus_handler = {
	.mouse_move = _mouse_move,
	.key_up = _key_up_cb
};

static void _key_up_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Up *ev)
{
	if (!data)
		return;

	if (!strcmp(ev->keyname, KEY_BACK) ||
			!strcmp(ev->keyname, KEY_ESC)) {
		inputmgr_remove_callback(data, &popup_handler);
		evas_object_del(data);
	}
}

static void _clicked(int id, void *data, Evas_Object *obj)
{
	notification_h noti;

	if (!data)
		return;

	switch (id) {
	case BUTTON_OK_NOTI_DEL:
		noti = evas_object_data_get(data, KEY_NOTI);
		if (noti)
			notification_delete(noti);
		break;
	case BUTTON_OK:
		evas_object_del(data);
		break;
	}
}

static void _up_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	int x, y, w, h;
	elm_scroller_region_get((Evas_Object*)data, &x, &y, &w, &h);
	elm_scroller_region_show((Evas_Object*)data, x, y-FONT_SIZE, w, h);
}

static void _down_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	int x, y, w, h;
	elm_scroller_region_get((Evas_Object*)data, &x, &y, &w, &h);
	elm_scroller_region_show((Evas_Object*)data, x, y+FONT_SIZE, w, h);
}

static void _ok_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_del((Evas_Object*)data);
}

static void _mouse_move(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev)
{
	elm_object_focus_set(obj, EINA_TRUE);
}

Evas_Object *util_show_notification_popup(
		Evas_Object *base, notification_h noti, bool noti_del)
{
	char *title = NULL;
	char *temp_content = NULL;
	char *content = NULL;

	char *start_tag = "<font=BreezeSans:style=Light color=#686868 align=left linesize=46 font_size=32>";
	char *end_tag = "</font>";

	Evas_Object *notification_popup = NULL;
	Evas_Object *window_layout = NULL;
	Evas_Object *up_btn, *down_btn, *ok_btn;
	Evas_Object *scroller, *scroller_label;

	Evas *e = NULL;
	Ecore_Evas *ee = NULL;
	Eina_Bool result = EINA_FALSE;
	int screen_x, screen_y, screen_w, screen_h;

	notification_get_text(noti, NOTIFICATION_TEXT_TYPE_TITLE, &title);
	notification_get_text(noti, NOTIFICATION_TEXT_TYPE_CONTENT, &temp_content);

	if (temp_content == NULL) {
		_ERR("temp_content is null");
		goto out;
	}

	int buffer_size = strlen(start_tag)+strlen(temp_content)+strlen(end_tag);

	content = (char *)calloc(buffer_size+1, sizeof(char));

	if (content == NULL) {
		_ERR("allocation for content failed");
		goto out;
	}
	snprintf(content, buffer_size, "%s%s%s", start_tag, temp_content, end_tag);

	/* Creating a Notification Window */
	notification_popup = elm_win_add(NULL, "notification_popup", ELM_WIN_BASIC);

	if (notification_popup == NULL) {
		_ERR("elm_win_add failed");
		goto out;
	}

	e = evas_object_evas_get(notification_popup);
	if (!e) {
		_ERR("evas_object_evas_get failed");
		evas_object_del(notification_popup);
		notification_popup = NULL;
		goto out;
	}

	ee = ecore_evas_ecore_evas_get(e);
	if (!ee) {
		_ERR("ecore_evas_ecore_evas_get failed");
		evas_object_del(notification_popup);
		notification_popup = NULL;
		goto out;
	}

	elm_win_alpha_set(notification_popup, EINA_TRUE);
	elm_win_borderless_set(notification_popup, EINA_TRUE);
	elm_win_autodel_set(notification_popup, EINA_TRUE);

	elm_win_screen_size_get(notification_popup, &screen_x, &screen_y, &screen_w, &screen_h);
	evas_object_resize(notification_popup, screen_w, screen_h);

	/* Layout */
	window_layout = elm_layout_add(notification_popup);
	result = elm_layout_file_set(window_layout, EDJEFILE, GRP_NOTIFICATION_POPUP);
	_DBG("Layout elm_layout_file_set for [%s] returns [%d]", EDJEFILE, result);
	evas_object_resize(window_layout, screen_w, screen_h);
	evas_object_show(window_layout);

	/* up, down btns */
	up_btn = elm_button_add(window_layout);
	elm_object_style_set(up_btn, STYLE_POPUP_UP_BTN);
	elm_object_focus_allow_set(up_btn, EINA_TRUE);
	elm_object_part_content_set(window_layout, "part_up_btn", up_btn);
	evas_object_show(up_btn);

	down_btn = elm_button_add(window_layout);
	elm_object_style_set(down_btn, STYLE_POPUP_DOWN_BTN);
	elm_object_focus_allow_set(down_btn, EINA_TRUE);
	elm_object_part_content_set(window_layout, "part_down_btn", down_btn);
	evas_object_show(down_btn);

	/* content - title */
	elm_object_part_text_set(window_layout, "part_title_label", title);

	/* content - scroller */
	scroller = elm_scroller_add(window_layout);
	elm_scroller_policy_set(scroller, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);
	elm_object_part_content_set(window_layout, "part_content_swallow", scroller);
	elm_object_scale_set(scroller, 1/edje_scale_get());
	evas_object_show(scroller);

	/* content - label in scroller */
	scroller_label = elm_label_add(scroller);
	elm_label_line_wrap_set(scroller_label, ELM_WRAP_WORD);
	elm_label_wrap_width_set(scroller_label, 563);
	elm_object_text_set(scroller_label, content);
	evas_object_show(scroller_label);

	elm_object_content_set(scroller, scroller_label);

	/* ok btn */
	ok_btn = elm_button_add(window_layout);
	elm_object_style_set(ok_btn, STYLE_POPUP_OK_BTN);
	elm_object_focus_allow_set(ok_btn, EINA_TRUE);
	elm_object_part_content_set(window_layout, "part_ok_btn", ok_btn);
	evas_object_show(ok_btn);

	evas_object_data_set(notification_popup, KEY_NOTI, noti);
	inputmgr_add_callback(notification_popup, 0, &popup_handler, notification_popup);
	inputmgr_add_callback(scroller, 0, &popup_handler, notification_popup);

	inputmgr_add_callback(ok_btn, 0, &focus_handler, notification_popup);
	inputmgr_add_callback(up_btn, 0, &focus_handler, notification_popup);
	inputmgr_add_callback(down_btn, 0, &focus_handler, notification_popup);

	/* register smart callback of buttons */
	evas_object_smart_callback_add(up_btn, "clicked", _up_btn_cb, scroller);
	evas_object_smart_callback_add(down_btn, "clicked", _down_btn_cb, scroller);
	evas_object_smart_callback_add(ok_btn, "clicked", _ok_btn_cb, notification_popup);

	evas_object_show(notification_popup);
	elm_win_activate(notification_popup);

out:
	if(content != NULL)
		free(content);

	return notification_popup;
}

void util_clear_notification(void)
{
	notification_list_h noti_list = NULL;
	notification_h noti;

	notification_get_list(NOTIFICATION_TYPE_NOTI, -1, &noti_list);
	if (!noti_list)
		return;

	while (noti_list != NULL) {
		noti = notification_list_get_data(noti_list);
		notification_delete(noti);

		noti_list = notification_list_remove(noti_list, noti);
		notification_free(noti);
	}
}

Evas_Object *util_add_icon(Evas_Object *parent, const char *file,
		const char *part)
{
	Evas_Object *ic;

	if (!parent) {
		_ERR("failed to get parent");
		return NULL;
	}

	ic = elm_icon_add(parent);
	if (!ic) {
		_ERR("failed to create icon");
		return NULL;
	}

	if (file)
		elm_image_file_set(ic, file, NULL);

	if (part)
		elm_object_part_content_set(parent, part, ic);

	elm_image_fill_outside_set(ic, EINA_TRUE);

	evas_object_show(ic);

	return ic;
}

Evas_Object *util_add_button(Evas_Object *parent, const char *part,
		const char *text, const char *style)
{
	Evas_Object *btn;

	if (!parent) {
		_ERR("Invalid argument.");
		return NULL;
	}

	btn = elm_button_add(parent);
	if (!btn) {
		_ERR("elm_button_add failed.");
		return NULL;
	}

	if (part)
		elm_object_part_content_set(parent, part, btn);
	if (text)
		elm_object_text_set(btn, text);
	if (style)
		elm_object_style_set(btn, style);

	evas_object_show(btn);

	return btn;
}

void util_set_last_viewed(void)
{
	int r;

	r = app_contents_set_basis_time(CONTENTS_NOTI);
	if (r != APP_CONTENTS_ERROR_NONE)
		_ERR("failed to set basis time");
}
